
var EDITOR_CONSTS = { USE_EXPANDABLE: true };

AssetManager = (function () {
    var convertSrcToUrlHandler = {
        name: function (asset) {
            asset.src = "assets/" + asset.src;
            return asset;
        },
        ordinalNumber: function (asset) {
            asset.src = EB.getAssetUrl('', asset.src);
            return asset;
        }
    };


    function setAssetUrlByJson(asset) {
        var ret = asset;
        if(!asset || !asset.hasOwnProperty("idProp")){
            return [{src:""}];
        }
        if(convertSrcToUrlHandler.hasOwnProperty(asset.idProp)) {
            ret = convertSrcToUrlHandler[asset.idProp](asset);
        }
        return ret;
    }

    function setAssetsUrlByElementProp (assets, prop){
        var targetAssets = assets.filter(function (asset){
            if (asset && asset.hasOwnProperty("elementProp") &&  asset.elementProp === prop) {
                return asset;
            }
        });
        if(targetAssets.length <= 0){
            return [{src:""}];
        }
        targetAssets.forEach(function (asset){
            asset.src = AssetManager.setAssetUrlByJson(asset).src;
        });
        return targetAssets;

    }

    return {
        setAssetUrlByJson: setAssetUrlByJson,
        setAssetsUrlByElementProp: setAssetsUrlByElementProp
    };

})();

// -------------------------- End Action Handler -------------------------

(function() {

    var insideAutoEvent = false;

    function factoryItemToAdkitItem(item) {

        var ans = { };

        function convertBase(item) {
            var result = {
                id : item.id,
                type : 'adkit-'+ item.type,
                width: item.width,
                height: item.height,
                top: item.Y,
                left: item.X,
                visible: item.visible,
                opacity: item.opacity,
                position: 'absolute',
                wordBreak: 'break-all',
                zIndex: item.Z
            };

            if (item.sv && item.sv.isEnabled && item.sv.svKey) {
                result.svKey = item.sv.svKey;
            }

            return result;
        }

        function convertHtml(item) {
            var result = convertBase(item);
            result.html = item.html;
            return result;
        }

        function convertImage(item) {
            var result = convertBase(item);
            if (item.assets.length) {
                result.image = item.assets[0].src;
            }
            return result;
        }

        function convertMedia(item){
            var result = convertBase(item);
            result.autoPlay = item.autoPlay || false;//autoplay
            result.autoPlayDelay = item.autoPlayDelay;//autoplay delay
            result.muted = item.muted || false; //muted
            result.showControls = item.controls || false; //controls
            
            return result;
        }

        function convertVideo(item){
            var result = convertMedia(item);

            result.mobileAutoPlay = item.mobileAutoplay;
            result.mobileAutoPlayDuration = item.mobileAutoplayDuration;

            result.videoSources = [];

            item.assets.forEach(function (asset){
                if(asset.elementProp === 'video'){//sources
                    result.videoSources.push(asset.src);
                }
                else if(asset.elementProp === 'videoPoster'){//poster
                    result.poster = asset.src;
                }
                else if(asset.elementProp === 'autoPlayTranscodedAsset'){//mobileAutoPlayAsset
                    result.mobileAutoPlayAsset = asset.src;
                }
            });

            return result;
        }

        function convertHotspot(item){
            var result = convertBase(item);

            return result;
        }

        function convertPinterest(item){
            var result = convertBase(item);

            result.shareUrl = item.advertiserURL;
            item.assets.forEach(function (asset){
                if(asset.elementProp === 'image'){
                    result.pinImage = asset.src;
                }
                else if(asset.elementProp === 'pinterestButton'){
                    result.buttonImage = asset.src;
                }
            });
            result.pinText = item.description;

            return result;
        }

        function convertTwitterShare(item){
            var result = convertBase(item);

            result.shareUrl = item.shareURL;
            result.buttonLayout = item.buttonLayout;
            result.tweetText = item.tweetText;
            result.viaUser = item.viaUser;
            result.recommendUser = item.recommendUser;
            result.hashTags = item.hashTags;

            return result;
        }

        function convertFacebookShare(item){
            var result = convertBase(item);

            result.shareUrl = item.shareURL;
            if(item.buttonLayout === 'icon' || item.buttonLayout === 'button'){
                result.buttonLayout = item.buttonLayout;
            }else if(item.buttonLayout === 'customImg'){    
                item.assets.forEach(function (asset){
                    if(asset.elementProp === 'facebookshareButton'){
                        result.buttonImage = asset.src;
                    }
                });
            }

            return result;
        }

        function convertYouTube(item){
            var result = convertBase(item);

            result.videoSource = item.videoId;
            result.showControls = item.showControlbar;
            result.showInfoBar = item.showInfobar;

            return result;
        }

        function convertFactoryButton(item){
            var result = convertBase(item);

            result.text = item.label;
            result.fontFamily = item.fontFamily;
            result.fontSize = item.fontSize;

            var convState = function convertButtonState(buttonState){
                var res = {
                    'type' : buttonState.type,
                    'borderStyle' : buttonState.buttonBorder,
                    'textStyle' : buttonState.buttonTextStyle,
                    'shadow' : buttonState.buttonShadow
                }

                res.backgroundStyle = {'type' : buttonState.buttonBackground.fillType};
                if('SolidColor' in buttonState.buttonBackground){
                    res.backgroundStyle.color = buttonState.buttonBackground.SolidColor;
                }
                if('gradientFirstColor' in buttonState.buttonBackground){
                    res.backgroundStyle.startColor = buttonState.buttonBackground.gradientFirstColor;
                }
                if('gradientSecondColor' in buttonState.buttonBackground){
                    res.backgroundStyle.endColor = buttonState.buttonBackground.gradientSecondColor;
                }
                if('gradientPercent' in buttonState.buttonBackground){
                    res.backgroundStyle.percent = buttonState.buttonBackground.gradientPercent;
                }

                item.assets.forEach(function (asset){
                    if(asset.elementProp === buttonState.type){
                        res.backgroundStyle.image = asset.src;
                    }
                });

                return res;
            };

            var normalStateStyle = convState(item.buttonState.normal);
            var hoverStateStyle = convState(item.buttonState.hover);
            var clickedStateStyle = convState(item.buttonState.clicked);

            result.stateStyle = [normalStateStyle, hoverStateStyle, clickedStateStyle];

            return result;
        }

        switch (item.type) {

            case 'html':
            case 'text':
                ans = convertHtml(item);
                break;
            case 'image':
                ans = convertImage(item);
                break;
            case 'video':
                ans = convertVideo(item);
                break;
            case 'hotspot':
                ans = convertHotspot(item);
                break;
            case 'twittershare':
                ans = convertTwitterShare(item);
                break;
            case 'facebookshare':
                ans = convertFacebookShare(item);
                break;
            case 'pinterest':
                ans = convertPinterest(item);
                break;
            case 'youtube':
                ans = convertYouTube(item);
                break;
            case 'custombutton':
                item.type = 'factoryButton';
                ans = convertFactoryButton(item);
                break;
        }

        return ans;
    }

    function run(params) {
        loadModules(params);
    }

    function loadFactoryModules(params, callback) {
        // load required runtime css and scripts
        var runtimeModules = ['interactions.js'];   
        var modulesLoaded = 0; // number of modules that finished loading
        var modulesToLoad = [];
    
        if (params.runtimeInfo[params.part].modules) {
            modulesToLoad = params.runtimeInfo[params.part].modules;
        }
  
        var numModules = modulesToLoad.length;

        if (numModules) {

            // script modules that finish loading dispatch the moduleLoadedEvent event so we can check if all modules were loaded and continue building the ad
            $("body").bind("moduleLoadedEvent", function () {
                modulesLoaded++;
                if (modulesLoaded === numModules)
                    callback();
            });
        
            for (var i in modulesToLoad) {
                var module = runtimeModules[modulesToLoad[i]];
                // get the module file extention to determine if this is a css of script file
                var fileExtension = module.substr(module.lastIndexOf(".")).toLowerCase();
                if (fileExtension == ".css") {
                    // load css using jQuery "get" in order to recieve a callback and continue building the ad when all modules are loaded
                    $.get("scripts/" + module, function (contents) {
                        $("<style type=\"text/css\">" + contents + "</style>").appendTo(document.head);
                        modulesLoaded++;
                        if (modulesLoaded == numModules)
                            callback();
                    });
                }
                else {
                    // load script. the script will dispatch the moduleLoadedEvent once loaded
                    var element = document.createElement('script');
                    element.setAttribute('type', 'text/javascript');
                    element.setAttribute('src', "scripts/" + module);
                    document.getElementsByTagName('head')[0].appendChild(element);
                }
            }
        }
        else {
            callback();
        }

    }

    function loadAdkitModules(params, callback) {
        if (params.runtimeInfo[params.part].adkitModules && params.runtimeInfo[params.part].adkitModules.length) {
            adkit.loadComponents(params.runtimeInfo[params.part].adkitModules, callback);
        }
        else{
            callback(params);
        }
    }

    function loadModules(params) {
        loadFactoryModules(params, function() {
            loadAdkitModules(params, function() {
                buildAd(params);
            });
        }); 
    }


    function buildAd(params) {

        adkit.setCurrentContainer('content');

        var jsonStr = '%7B%22name%22%3A%22PL-PL_WOW_Promo_100M_HTML5_728x90_noloop%5Ct%5Ct%5Ct%5Ct%5Ct%5Ct%22%2C%22adID%22%3A%22%22%2C%22format%22%3A%22Polite%22%2C%22ct%22%3A%22%22%2C%22panels%22%3A%5B%7B%22type%22%3A%22panel%22%2C%22id%22%3A%22Banner%22%2C%22platformId%22%3A%22%22%2C%22width%22%3A728%2C%22height%22%3A90%2C%22X%22%3A0%2C%22Y%22%3A0%2C%22interactions%22%3A%5B%7B%22eventId%22%3A%22adLoad%22%2C%22actions%22%3A%5B%7B%22actionId%22%3A%22animation.hide%22%2C%22actionParams%22%3A%7B%22version%22%3A%221.0.0%22%2C%22delay%22%3A0%7D%2C%22actionTarget%22%3A%22hotspot2%22%2C%22actionOrder%22%3A%7B%22index%22%3A0%2C%22sequenceState%22%3A%22onTrigger%22%2C%22extraDelay%22%3A0%7D%7D%5D%7D%5D%2C%22defaultImage%22%3A%22OnGoing_100M_728x90_PL.jpg%22%2C%22usebackgroundcolor%22%3Atrue%2C%22backgroundcolor%22%3A%22%23000000%22%2C%22posType%22%3A1%2C%22delayedExpansion%22%3Afalse%2C%22autoExpand%22%3Afalse%2C%22autoRetract%22%3A2%2C%22assets%22%3A%5B%7B%22src%22%3A%22OnGoing_100M_728x90_PL.jpg%22%2C%22idProp%22%3A%22name%22%2C%22elementProp%22%3A%22defaultImage%22%7D%5D%2C%22scenes%22%3A%5B%7B%22name%22%3A%22Scene1%22%2C%22isDefault%22%3Atrue%2C%22elements%22%3A%5B%7B%22type%22%3A%22video%22%2C%22id%22%3A%22mp4_WoW_6-2%22%2C%22width%22%3A728%2C%22height%22%3A90%2C%22opacity%22%3A1%2C%22X%22%3A0%2C%22Y%22%3A0%2C%22Z%22%3A1%2C%22hidden%22%3Afalse%2C%22locked%22%3Atrue%2C%22interactions%22%3A%5B%7B%22eventId%22%3A%22play%22%2C%22actions%22%3A%5B%7B%22actionId%22%3A%22animation.hide%22%2C%22actionParams%22%3A%7B%22version%22%3A%221.0.0%22%2C%22delay%22%3A0%7D%2C%22actionTarget%22%3A%22hotspot2%22%2C%22actionOrder%22%3A%7B%22index%22%3A0%2C%22sequenceState%22%3A%22onTrigger%22%2C%22extraDelay%22%3A0%7D%7D%5D%7D%2C%7B%22eventId%22%3A%22ended%22%2C%22actions%22%3A%5B%7B%22actionId%22%3A%22animation.show%22%2C%22actionParams%22%3A%7B%22version%22%3A%221.0.0%22%2C%22delay%22%3A0%7D%2C%22actionTarget%22%3A%22hotspot2%22%2C%22actionOrder%22%3A%7B%22index%22%3A0%2C%22sequenceState%22%3A%22onTrigger%22%2C%22extraDelay%22%3A0%7D%7D%5D%7D%5D%2C%22assets%22%3A%5B%7B%22src%22%3A%22https%3A//blzadeu-a.akamaihd.net/banners/mpu/1007_100M/OnGoing_728x90_PL_apr14.mp4%22%2C%22idProp%22%3A%22url%22%2C%22elementProp%22%3A%22video%22%7D%5D%2C%22sv%22%3A%7B%7D%2C%22controls%22%3Afalse%2C%22muted%22%3Atrue%2C%22autoPlay%22%3Atrue%2C%22autoPlayDelay%22%3A0%2C%22visible%22%3Atrue%2C%22mobileAutoplay%22%3Afalse%2C%22mobileAutoplayDuration%22%3A0%7D%2C%7B%22type%22%3A%22image%22%2C%22id%22%3A%22Border%22%2C%22width%22%3A728%2C%22height%22%3A90%2C%22opacity%22%3A1%2C%22X%22%3A0%2C%22Y%22%3A0%2C%22Z%22%3A2%2C%22hidden%22%3Atrue%2C%22locked%22%3Atrue%2C%22interactions%22%3A%5B%5D%2C%22assets%22%3A%5B%7B%22src%22%3A%22728x90_border.png%22%2C%22idProp%22%3A%22name%22%2C%22elementProp%22%3A%22image%22%7D%5D%2C%22sv%22%3A%7B%22isEnabled%22%3Afalse%2C%22linkedTo%22%3A%22image1%22%2C%22svKey%22%3A%22Border%22%7D%2C%22visible%22%3Atrue%7D%2C%7B%22type%22%3A%22hotspot%22%2C%22id%22%3A%22hotspot1%22%2C%22width%22%3A728%2C%22height%22%3A90%2C%22opacity%22%3A1%2C%22X%22%3A0%2C%22Y%22%3A0%2C%22Z%22%3A3%2C%22hidden%22%3Atrue%2C%22locked%22%3Atrue%2C%22interactions%22%3A%5B%7B%22eventId%22%3A%22click%22%2C%22actions%22%3A%5B%7B%22actionId%22%3A%22urlActions.triggerCT%22%2C%22actionParams%22%3A%7B%22version%22%3A%221.0.0%22%2C%22delay%22%3A0%7D%2C%22actionTarget%22%3A%22%22%2C%22actionOrder%22%3A%7B%22index%22%3A0%2C%22sequenceState%22%3A%22onTrigger%22%2C%22extraDelay%22%3A0%7D%7D%2C%7B%22actionId%22%3A%22%22%2C%22actionParams%22%3Anull%2C%22actionTarget%22%3A%22%22%2C%22actionOrder%22%3A%7B%22index%22%3A1%2C%22sequenceState%22%3A%22after%22%2C%22extraDelay%22%3A0%7D%7D%2C%7B%22actionId%22%3A%22%22%2C%22actionParams%22%3Anull%2C%22actionTarget%22%3A%22%22%2C%22actionOrder%22%3A%7B%22index%22%3A2%2C%22sequenceState%22%3A%22after%22%2C%22extraDelay%22%3A0%7D%7D%5D%7D%5D%2C%22assets%22%3A%5B%5D%2C%22sv%22%3A%7B%7D%7D%5D%7D%5D%7D%5D%2C%22bannerIndex%22%3A0%2C%22autoDraftSet%22%3Afalse%2C%22UpdateDate%22%3A1435337254301%2C%22fonts%22%3A%7B%22version%22%3A1%2C%22fontList%22%3A%5B%7B%22name%22%3A%22Titillium%20Web%22%2C%22code%22%3A%22%3Clink%20href%3D%27//fonts.googleapis.com/css%3Ffamily%3DTitillium+Web%27%20rel%3D%27stylesheet%27%20type%3D%27text/css%27%3E%22%2C%22alternate%22%3A%22Arial%22%2C%22inUse%22%3Afalse%7D%5D%7D%2C%22externalAssets%22%3A%5B%7B%22name%22%3A%22OnGoing_728x90_PL_apr14.mp4%22%2C%22url%22%3A%22https%3A//blzadeu-a.akamaihd.net/banners/mpu/1007_100M/OnGoing_728x90_PL_apr14.mp4%22%2C%22isExternal%22%3Atrue%7D%5D%2C%22assetsToFilter%22%3A%7B%22extensions%22%3A%5B%22html%22%2C%22css%22%2C%22js%22%5D%2C%22names%22%3A%5B%22defaultdb2768465ffe462d94706cfcf9ede5f2.jpeg%22%5D%7D%2C%22version%22%3A%221.0.7%22%7DMMAdJSON';
        jsonStr = jsonStr.replace("MMAdJSON", "");
        var data = $.parseJSON(unescape(jsonStr));
      
        if (params.runtimeInfo[params.part].fonts) {
            var fontsToLoad = params.runtimeInfo[params.part].fonts;
            // load fonts in use
            if (fontsToLoad.length && data["fonts"]) {
                FontManager.createFromJSON(data.fonts, fontsToLoad);
            }
        }

        // define event handlers for clickthrougth and custom interaction events
        $(window).bind("clickThroughEvent", handleClickThrough);
        $(window).bind("customInteractionEvent", handleCustomInteraction);
        $(window).bind("elementHidden", handleElementHidden);
        $(window).bind("elementFadeOut", handleElementFadeOut);

        if (data.format == "SingleExpandable" || data.format == "Expandable") {
            $(window).bind("collapseEvent", data, handleCollapse);
            $(window).bind("expandEvent", data, handleExpand);
            $(window).bind("closeAllAdPartsEvent", handleCloseAllAdParts);
        }

        // get reference to the panel we should draw
        //var panel = data.panels[params.part];
        var panel = data.panels[params.part];

        if (panel.usebackgroundcolor) {
            $("#content").css("background-color", panel.backgroundcolor);
        }

        var i1, i2;
        var content = "";
  

        for (i1 in panel.scenes[0].elements) {
            var element = panel.scenes[0].elements[i1];
            if (isSupportedByAdKit(element.type)){
                adkit.addItem(factoryItemToAdkitItem(element));
            } else {
                content += window[element.type.toLowerCase()].getHTML(element); // draw the element
            }   
        }

        // add the ad div to the page
        $('#content').append(content);
    
        var interactionList = [];

        // add actions
        if (panel.interactions) {
            var interactions = panel.interactions;
            for (i1 in interactions) {
                var interaction = Interaction.createFromJSON(interactions[i1]);
                interaction.init();
                interaction.attach('content');
                interactionList.push(interaction);
            }
        }

        // initialize objects after added to DOM (if needed)
        for (i1 in panel.scenes[0].elements) {
            var element = panel.scenes[0].elements[i1];
            if (!isSupportedByAdKit(element.type)){
                window[element.type.toLowerCase()].init(element);
            }
        }

        // add actions
        for (i1 in panel.scenes[0].elements) {
            var element = panel.scenes[0].elements[i1];
            var isElementClickable = false;
            if (element.interactions) {
                var interactions = element.interactions;
                for (i2 in interactions) {
                    var interaction = Interaction.createFromJSON(interactions[i2]);
                    interaction.init();
                    interaction.attach(element.id);
                    interactionList.push(interaction);

                    if (interactions[i2].eventId === "click" || interactions[i2].eventId === "dblclick") {
                        isElementClickable = true;
                    }
                }
            }
           if (isElementClickable) {
                $("#" + element.id).css("cursor", "pointer");
            }

        }

        for (i1 in interactionList) {
            interactionList[i1].prepare();
        }

        insideAutoEvent = (data.bannerIndex === params.part);
        $('#content').swipe(handleSwipe); 
        $(window).on('shake', function(e) {
            $('#content').trigger(e);
        }); 
        $('#content').trigger('adLoad');
        insideAutoEvent = false;
    }

    // handle clickThrough event
    function handleClickThrough(e) {
        EB.clickthrough();
    }

    function handleSwipe(e) {
        EB.userSwipe();
    }

    function handleExpand(e, target) {
        Utils.stopPropagation(e);
        if (e.data.format === "Expandable") {
            EB.expand({panelName: target, actionType: insideAutoEvent ? "auto" : "user"});
        }
        else if (e.data.format === "SingleExpandable") {
            EB.expand({actionType: insideAutoEvent ? "auto" : "user"});
        }
        $('#content').trigger('expandInterEvent');
    }

    function handleCollapse(e, target) {
        Utils.stopPropagation(e);
        if (e.data.format === "Expandable") {
            EB.collapse({panelName: target});
        }
        else if (e.data.format === "SingleExpandable") {
            EB.collapse();
            var collapseRect = e.data.collapse.split(",");
            var clipRect = {
                top: parseInt(collapseRect[0]),
                left: parseInt(collapseRect[1]),
                right: parseInt(collapseRect[1]) + parseInt(collapseRect[2]),
                bottom: parseInt(collapseRect[0]) + parseInt(collapseRect[3])
            }
            pauseVideosInCollapsedArea(clipRect);
        }
   
        $('#content').trigger('collapseInterEvent');
    }

    function handleCloseAllAdParts(e) {
        EB.collapse();
    }

    function handleCustomInteraction(e, name) {
        Utils.stopPropagation(e);
        if (name) {
            EB.userActionCounter(name);
        }
    }

    function handleElementHidden(e, t) {
        Utils.stopPropagation(e);
        pauseVideoByDivId(t);
    }

    function handleElementFadeOut(e, t) {
        Utils.stopPropagation(e);
        pauseVideoByDivId(t);
    }

    function pauseVideoByDivId(divId){
        var pauseElemByIdFunc = function(type)
        {
            var list = getComponentsByType(type);
            if(list){
                list.forEach(function (elem) {
                    if(elem.div.id === divId){
                        elem.pause();
                        return;
                    }
                });
            }
        };

        pauseElemByIdFunc('adkit-video');
        pauseElemByIdFunc('adkit-youtube');
    }

    function getComponentsByType(type){
        return adkit.getComponentsByType(type);
    }

    /**
     * Run over all videos components (video and youtube components) and pause them if they are in the collapsed area
     * @param clipRect
     */
    function pauseVideosInCollapsedArea(clipRect) {
        var pauseVideoInListFunc = function(type){
            var list = getComponentsByType(type);
            if(list){
                list.forEach(function (elem) {
                    if(isPauseVideoNeeded(clipRect, elem.div.id)){
                        elem.pause();
                    }
                });
            }
        };

        pauseVideoInListFunc('adkit-video');
        pauseVideoInListFunc('adkit-youtube');
    }

    /**
     * Throw pauseVideo event if our element (the second input parameter) is intersect with the clip rectangle (the first input parameter)
     * @param clipRect
     * @param elemId
     */
    function isPauseVideoNeeded(clipRect, elemId) {
        return !getIntersection(clipRect, getElementRect(elemId)).intersect;
    }

    function getElementRect(elemId) {
        return document.getElementById(elemId).getBoundingClientRect();
    }

    /**
     * Check if two rectangles are intersected
     * @param rect1
     * @param rect2
     * @returns Intersect rectangle
     */
    function getIntersection(rect1, rect2) {
        var result = { };
        result.intersect = !(rect1.right < rect2.left || rect2.right < rect1.left || rect1.bottom < rect2.top || rect2.bottom < rect1.top);
        if (result.intersect)
        {
            var rect = { };
            rect.left = Math.max(rect1.left, rect2.left);
            rect.top = Math.max(rect1.top, rect2.top);
            rect.right = Math.min(rect1.right, rect2.right);
            rect.bottom = Math.min(rect1.bottom, rect2.bottom);
            result.rect = rect;
        }
        return result;
    }

    function isSupportedByAdKit(type){
        switch(type){
            case 'text':
            case 'html':
            case 'image':
            case 'video':
            case 'hotspot':
            case 'twittershare':
            case 'facebookshare':
            case 'pinterest':
            case 'youtube':
            case 'factoryButton'://after we change the type of custombutton to factoryButton
            case 'custombutton':
                return true;
            default:
                return false;
        }
    }

    adkit.onReady(function() {
        onInit(adkit.getServingParams());
    });

    function onInit(params) {var runtimeInfo = [{"modules":[0],"adkitModules":["adkit-video","adkit-image","adkit-hotspot"]}]; if (params) { var cparam = $.parseJSON(unescape(params)); cparam.runtimeInfo = runtimeInfo; run(cparam); } else {run({part: 0, runtimeInfo: runtimeInfo}); }      if (false) {}}

})();


